#paused {
	font-size:12px;
	color:#666666;
	display:none;
}
#exposed_functions table td {
	text-align:right;
}
#exposed_functions table td li {
	direction: rtl;
}
#exposed_functions table td, #exposed_functions table th {
	vertical-align:top;
}
#request_data {
    width:auto;
}
#request_data th, #request_data td {
    margin:0;
    padding:2px;
}
#pending_requests, #active_requests {
    font-size:14px;  
}
#request_data th {
    text-align:right;
    font-size:14px;
}
#request_data td {
    text-align:left;
}